<?php
namespace Ak\Mail;

use Ak\Lib\File\File;
use Ak\Lib\File\FileInterface;
use Laminas\Mail\Header\ContentType;
use Laminas\Mail\Message as BaseMessage;
use Laminas\Mime\Message as MimeMessage;
use Laminas\Mime\Mime;
use Laminas\Mime\Part as MimePart;

/**
 * E-mail poruka.
 *
 * @package Ak\Mail
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/mail
 */
final class Message extends BaseMessage
{
    /** @var string */
    protected $encoding = 'utf-8';

    private ?MimePart $textPart = null;
    private ?MimePart $htmlPart = null;
    private ?MimeMessage $relatedContent = null;
    private ?MimeMessage $alternativeContent = null;
    private ?MimeMessage $mixedContent = null;

    /**
     * Postavlja tekstualni sadržaj poruke.
     *
     * @param string $body
     */
    private function setTextBody(string $body): void
    {
        if (null === $this->textPart) {
            $this->textPart = new MimePart($body);
            $this->textPart->setType(Mime::TYPE_TEXT);
            $this->textPart->setEncoding(Mime::ENCODING_QUOTEDPRINTABLE);
            $this->textPart->setCharset('utf-8');
        } else {
            $this->textPart->setContent($body);
        }

        $this->setAlternativePart($this->textPart);
    }

    /**
     * Postavlja HTML sadržaj poruke.
     *
     * @param string $body
     */
    private function setHtmlBody(string $body): void
    {
        if (null === $this->htmlPart) {
            $this->htmlPart = new MimePart($body);
            $this->htmlPart->setType(Mime::TYPE_HTML);
            $this->htmlPart->setEncoding(Mime::ENCODING_QUOTEDPRINTABLE);
            $this->htmlPart->setCharset('utf-8');
        } else {
            $this->htmlPart->setContent($body);
        }

        if (preg_match_all('/<img[^>]+src="([^"]+)"/', $body, $matches)) {
            $this->setHtmlBodyWithInlineImages(array_unique($matches[1]));
            return;
        } else {
            $this->relatedContent = null;
        }

        $this->setAlternativePart($this->htmlPart);
    }

    /**
     * Postavlja HTML sadržaj koji sadrži inline slike.
     *
     * @param array $imagePaths
     */
    private function setHtmlBodyWithInlineImages(array $imagePaths = [])
    {
        $this->relatedContent = new MimeMessage();
        $this->getHeaders()->removeHeader('Content-Type');

        $cids = [];
        $parts = [];

        foreach ($imagePaths as $imagePath) {
            if (!file_exists($imagePath)) {
                continue;
            }

            $image = new File(fopen($imagePath, 'r'));
            $imageCid = sprintf('cid_%s', $image->getMd5Hash());

            $imagePart = new MimePart(fopen($imagePath, 'r'));
            $imagePart->setId($imageCid);
            $imagePart->setType($image->getMimeType());
            $imagePart->setDisposition(Mime::DISPOSITION_INLINE);
            $imagePart->setEncoding(Mime::ENCODING_BASE64);

            $cids[$imagePath] = $imageCid;
            $parts[] = $imagePart;
        }

        $regexSearch = [];
        $regexReplace = [];

        foreach ($cids as $path => $cid) {
            $regexSearch[] = sprintf('/src="%s"/', preg_quote($path, '/'));
            $regexReplace[] = sprintf('src="cid:%s"', $cid);
        }

        $html = preg_replace($regexSearch, $regexReplace, $this->htmlPart->getRawContent());
        $this->htmlPart->setContent($html);

        array_unshift($parts, $this->htmlPart);
        $this->relatedContent->setParts($parts);

        $this->setAlternativePart($this->relatedContent);
    }

    /**
     * Postavlja jedan od alternativnih dijelova poruke.
     *
     * @param MimePart|MimeMessage $part
     */
    private function setAlternativePart($part)
    {
        if (null === $this->body) {
            $this->setBody($part);
            return;
        }

        if (null === $this->alternativeContent) {
            $this->alternativeContent = new MimeMessage();
            $this->alternativeContent->setParts($this->body->getParts());
        }

        if (count($this->alternativeContent->getParts()) === 0) {
            $this->setBody($part);
            return;
        }

        if ($part === $this->relatedContent) {
            $part = $this->getRelatedPart();
        }

        switch ($part->getType()) {
            case Mime::TYPE_HTML:
                $parts = [$this->textPart, $part];
                break;
            case Mime::TYPE_TEXT:
                $parts = [$part, $this->getRelatedPart()];
                break;
            default:
                $parts = [$this->textPart, $part];
        }

        $parts = array_filter($parts);

        if (count($parts) === 1) {
            $this->setBody($parts[0]);
            return;
        }

        $this->alternativeContent->setParts($parts);
        $this->setBody($this->alternativeContent);
    }

    private function getRelatedPart(): ?MimePart
    {
        if (null === $this->relatedContent) {
            return $this->htmlPart ?? null;
        }

        $content = $this->relatedContent->generateMessage();

        if (preg_match('/--(=_[a-z\d]{32})/', $content, $matches)) {
            $boundary = $matches[1];
        }

        $part = new MimePart($content);
        $part->setType(Mime::MULTIPART_RELATED);
        $part->setBoundary($boundary ?? null);

        return $part;
    }

    private function getAlternativePart(): ?MimePart
    {
        if (null === $this->alternativeContent) {
            return null;
        }

        $content = $this->alternativeContent->generateMessage();

        if (preg_match('/--(=_[a-z\d]{32})/', $content, $matches)) {
            $boundary = $matches[1];
        }

        $part = new MimePart($content);
        $part->setType(Mime::MULTIPART_ALTERNATIVE);
        $part->setBoundary($boundary ?? null);

        return $part;
    }

    /**
     * Dodaje privitak u ovu e-mail poruku.
     *
     * @param FileInterface $file
     */
    public function addAttachment(FileInterface $file): void
    {
        if (null === $this->mixedContent) {
            $this->mixedContent = new MimeMessage();

            if ($this->body === $this->alternativeContent) {
                $this->mixedContent->setParts([$this->getAlternativePart()]);
            } else if ($this->body === $this->relatedContent) {
                $this->mixedContent->setParts([$this->getRelatedPart()]);
            } else {
                $this->mixedContent->setParts($this->body->getParts());
            }
        }

        $attachment = new MimePart($file->getContents());
        $attachment->setType($file->getMimeType());
        $attachment->setFileName($file->getName());
        $attachment->setEncoding(Mime::ENCODING_BASE64);
        $attachment->setDisposition(Mime::DISPOSITION_ATTACHMENT);

        $this->mixedContent->addPart($attachment);
        $this->getHeaders()->removeHeader('Content-Type');

        $this->setBody($this->mixedContent);
    }

    /**
     * Postavlja tijelo poruke.
     *
     * Naša verzija omogućava dodavanje alternativnih verzija.
     *
     * @param object|string|MimeMessage|null $body
     *
     * @return $this|BaseMessage
     */
    public function setBody($body)
    {
        if (is_string($body)) {
            if (strip_tags($body) !== $body) {
                $this->setHtmlBody($body);
            } else {
                $this->setTextBody($body);
            }
            return $this;
        }

        if ($body instanceof MimePart) {
            $message = new MimeMessage();
            $message->setParts([$body]);
            return $this->setBody($message);
        }

        parent::setBody($body);

        if ($this->body instanceof MimeMessage) {
            /** @var ContentType $contentType */
            $contentType = $this->getHeaderByName('Content-Type', ContentType::class);

            if ($this->body->isMultiPart()) {
                if ($this->body === $this->relatedContent) {
                    $contentType->setType(Mime::MULTIPART_RELATED);
                } else if ($this->body === $this->alternativeContent) {
                    $contentType->setType(Mime::MULTIPART_ALTERNATIVE);
                }
            }
        }

        return $this;
    }
}
