<?php
namespace AkTest\Mail;

use Ak\Lib\File\File;
use Ak\Mail\Message;
use Ak\Test\AbstractTestCase;
use Laminas\Mail\Header\ContentType;
use Laminas\Mail\Transport\TransportInterface;
use Laminas\Mime\Message as MimeMessage;
use Psr\Container\ContainerExceptionInterface;
use Psr\Container\NotFoundExceptionInterface;

/**
 * Class MessageTest
 *
 * @package AkTest\Mail
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/mail
 */
final class MessageTest extends AbstractTestCase
{
    public $configProviders = [
        TestConfigProvider::class,
    ];

    private function prepareMessage(Message $message, string $subject = null)
    {
        $message->setFrom('noreply@carparts.hr', 'Pošiljatelj đšžćč');
        $message->setTo('hipnaba@gmail.com', 'Primatelj đšžćč');
        $message->setSubject($subject ?? 'Test message');
    }

    /**
     * @param Message $message
     * @param string|null $subject
     *
     * @throws ContainerExceptionInterface
     * @throws NotFoundExceptionInterface
     */
    private function sendMessage(Message $message, string $subject = null)
    {
        $this->prepareMessage($message, $subject);

        $container = $this->getContainer();
        /** @var TransportInterface $transport */
        $transport = $container->get('mail.sendgrid');
        $transport->send($message);
    }

    /**
     * @throws ContainerExceptionInterface
     * @throws NotFoundExceptionInterface
     */
    public function testCanSendPlainTextMessages()
    {
        $message = new Message();
        $message->setBody('nekakav sadržaj đšžćč');

        $headers = $message->getHeaders();

        /** @var ContentType $contentType */
        $contentType = $headers->get('Content-Type');
        $this->assertEquals('text/plain', $contentType->getType());

        /** @var MimeMessage $body */
        $body = $message->getBody();

        $this->assertInstanceOf(MimeMessage::class, $body);
        $this->assertStringContainsString('nekakav sadržaj đšžćč', quoted_printable_decode($body->generateMessage()));

        if (TestConfigProvider::SEND_EMAILS) {
            $this->sendMessage($message, __METHOD__ . ' -- đšžćč');
        }
    }

    /**
     * @throws ContainerExceptionInterface
     * @throws NotFoundExceptionInterface
     */
    public function testCanSendPlainTextMessagesWithAttachments()
    {
        $message = new Message();
        $message->setBody('nekakav sadržaj đšžćč');

        $image = new File(fopen(__DIR__ . '/data/flowers.jpg', 'r'), 'flowers.jpg');
        $message->addAttachment($image);

        $headers = $message->getHeaders();

        /** @var ContentType $contentType */
        $contentType = $headers->get('Content-Type');
        $this->assertEquals('multipart/mixed', $contentType->getType());

        /** @var MimeMessage $body */
        $body = $message->getBody();
        $parts = array_values($body->getParts());

        $this->assertCount(2, $parts);
        $this->assertEquals('text/plain', $parts[0]->getType());
        $this->assertEquals($image->getMimeType(), $parts[1]->getType());

        if (TestConfigProvider::SEND_EMAILS) {
            $this->sendMessage($message, __METHOD__);
        }
    }

    /**
     * @throws ContainerExceptionInterface
     * @throws NotFoundExceptionInterface
     */
    public function testCanSendHtmlMessages()
    {
        $message = new Message();
        $message->setBody('<p>nekakav sadržaj đšžćč</p>');

        $headers = $message->getHeaders();

        /** @var ContentType $contentType */
        $contentType = $headers->get('Content-Type');
        $this->assertInstanceOf(ContentType::class, $contentType);
        $this->assertEquals('text/html', $contentType->getType());

        $body = $message->getBody();

        $this->assertInstanceOf(MimeMessage::class, $body);
        $this->assertStringContainsString('<p>nekakav sadržaj đšžćč</p>', quoted_printable_decode($body->generateMessage()));

        if (TestConfigProvider::SEND_EMAILS) {
            $this->sendMessage($message, __METHOD__);
        }
    }

    /**
     * @throws ContainerExceptionInterface
     * @throws NotFoundExceptionInterface
     */
    public function testCanSendHtmlMessagesWithAttachments()
    {
        $message = new Message();
        $message->setBody('<p>nekakav sadržaj đšžćč</p>');

        $image = new File(fopen(__DIR__ . '/data/flowers.jpg', 'r'), 'flowers.jpg');
        $message->addAttachment($image);
        $spreadsheet = new File(fopen(__DIR__ . '/data/spreadsheet.xlsx', 'r'), 'spreadsheet.xlsx');
        $message->addAttachment($spreadsheet);

        $headers = $message->getHeaders();

        /** @var ContentType $contentType */
        $contentType = $headers->get('Content-Type');
        $this->assertInstanceOf(ContentType::class, $contentType);
        $this->assertEquals('multipart/mixed', $contentType->getType());

        $body = $message->getBody();
        $parts = array_values($body->getParts());

        $this->assertCount(3, $parts);
        $this->assertEquals('text/html', $parts[0]->getType());
        $this->assertEquals($image->getMimeType(), $parts[1]->getType());
        $this->assertEquals($spreadsheet->getMimeType(), $parts[2]->getType());

        if (TestConfigProvider::SEND_EMAILS) {
            $this->sendMessage($message, __METHOD__);
        }
    }

    /**
     * @throws ContainerExceptionInterface
     * @throws NotFoundExceptionInterface
     */
    public function testCanSendHtmlWithInlineImagesMessages()
    {
        $flowers = __DIR__ . '/data/flowers.jpg';
        $obama = __DIR__ . '/data/obama.jpg';

        $html = <<< EOH
<table>
    <tbody>
        <tr>
            <td>
                <img src="$obama" width="200" alt=""><br>
                Neka slika
            </td>
            
            <td style="width:100%">
                Prvi red
                <img src="$flowers" style="text-align:right;" width="300" alt="">
            </td>
        </tr>
        
        <tr>            
            <td>
                Drugi red
                <img src="$flowers" width="300" style="text-align:right;" alt="">
            </td>
            
            <td>
                <img src="$obama" width="200" alt=""><br>
                Neka slika
            </td>
        </tr>
    </tbody>
</table>
EOH;

        $message = new Message();
        $message->setBody($html);
        $headers = $message->getHeaders();

        /** @var ContentType $contentType */
        $contentType = $headers->get('Content-Type');
        $this->assertInstanceOf(ContentType::class, $contentType);
        $this->assertEquals('multipart/related', $contentType->getType());

        $body = $message->getBody();
        $parts = array_values($body->getParts());

        $this->assertCount(3, $parts);

        if (TestConfigProvider::SEND_EMAILS) {
            $this->sendMessage($message, __METHOD__);
        }
    }

    /**
     * @throws ContainerExceptionInterface
     * @throws NotFoundExceptionInterface
     */
    public function testCanSendHtmlWithInlineImagesMessagesAndAttachments()
    {
        $flowers = __DIR__ . '/data/flowers.jpg';
        $obama = __DIR__ . '/data/obama.jpg';

        $html = <<< EOH
<table>
    <tbody>
        <tr>
            <td>
                <img src="$obama" width="200" alt=""><br>
                Neka slika
            </td>
            
            <td style="width: 100%">
                Prvi red
                <img src="$flowers" width="300" style="text-align:right;" alt="">
            </td>
        </tr>
        
        <tr>            
            <td>
                Drugi red
                <img src="$flowers" width="300" style="text-align:right;" alt="">
            </td>
            
            <td>
                <img src="$obama" width="200" alt=""><br>
                Neka slika
            </td>
        </tr>
    </tbody>
</table>
EOH;

        $message = new Message();
        $message->setBody($html);

        $spreadsheet = new File(fopen(__DIR__ . '/data/spreadsheet.xlsx', 'r'), 'spreadsheet.xlsx');
        $message->addAttachment($spreadsheet);

        $headers = $message->getHeaders();

        /** @var ContentType $contentType */
        $contentType = $headers->get('Content-Type');
        $this->assertEquals('multipart/mixed', $contentType->getType());

        $body = $message->getBody();
        $parts = array_values($body->getParts());

        $this->assertCount(2, $parts);
        $this->assertEquals('multipart/related', $parts[0]->getType());
        $this->assertEquals($spreadsheet->getMimeType(), $parts[1]->getType());

        if (TestConfigProvider::SEND_EMAILS) {
            $this->sendMessage($message, __METHOD__);
        }
    }

    /**
     * @throws ContainerExceptionInterface
     * @throws NotFoundExceptionInterface
     */
    public function testCanSendAlternativeMessages()
    {
        $message = new Message();
        $message->setBody('nekakav sadržaj đšžćč');
        $message->setBody('<p>nekakav sadržaj đšžćč</p>');

        $headers = $message->getHeaders();

        /** @var ContentType $contentType */
        $contentType = $headers->get('Content-Type');
        $this->assertInstanceOf(ContentType::class, $contentType);
        $this->assertEquals('multipart/alternative', $contentType->getType());

        $body = $message->getBody();
        $parts = array_values($body->getParts());

        $this->assertCount(2, $parts);

        if (TestConfigProvider::SEND_EMAILS) {
            $this->sendMessage($message, __METHOD__);
        }
    }

    /**
     * @throws ContainerExceptionInterface
     * @throws NotFoundExceptionInterface
     */
    public function testCanSendAlternativeMessagesWithAttachments()
    {
        $message = new Message();
        $message->setBody('običan tekst');
        $message->setBody('<p>nekakav sadržaj đšžćč</p>');

        $image = new File(fopen(__DIR__ . '/data/flowers.jpg', 'r'), 'flowers.jpg');
        $message->addAttachment($image);
        $spreadsheet = new File(fopen(__DIR__ . '/data/spreadsheet.xlsx', 'r'), 'spreadsheet.xlsx');
        $message->addAttachment($spreadsheet);

        $headers = $message->getHeaders();

        /** @var ContentType $contentType */
        $contentType = $headers->get('Content-Type');
        $this->assertEquals('multipart/mixed', $contentType->getType());

        $body = $message->getBody();
        $parts = array_values($body->getParts());

        $this->assertCount(3, $parts);
        $this->assertEquals('multipart/alternative', $parts[0]->getType());
        $this->assertEquals($image->getMimeType(), $parts[1]->getType());
        $this->assertEquals($spreadsheet->getMimeType(), $parts[2]->getType());

        if (TestConfigProvider::SEND_EMAILS) {
            $this->sendMessage($message, __METHOD__);
        }
    }

    /**
     * @throws ContainerExceptionInterface
     * @throws NotFoundExceptionInterface
     */
    public function testCanSendAlternativeMessagesWithHtmlWithInlineImages()
    {
        $flowers = __DIR__ . '/data/flowers.jpg';
        $obama = __DIR__ . '/data/obama.jpg';

        $html = <<< EOH
<table>
    <tbody>
        <tr>
            <td>
                <img src="$obama" width="200" alt=""><br>
                Neka slika
            </td>
            
            <td style="width:100%">
                Prvi red
                <img src="$flowers" width="300" style="text-align:right;" alt="">
            </td>
        </tr>
        
        <tr>            
            <td>
                Drugi red
                <img src="$flowers" width="300" style="text-align:right;" alt="">
            </td>
            
            <td>
                <img src="$obama" width="200" alt=""><br>
                Neka slika
            </td>
        </tr>
    </tbody>
</table>
EOH;

        $message = new Message();
        $message->setBody('običan tekst');
        $message->setBody($html);

        $headers = $message->getHeaders();

        /** @var ContentType $contentType */
        $contentType = $headers->get('Content-Type');
        $this->assertInstanceOf(ContentType::class, $contentType);
        $this->assertEquals('multipart/alternative', $contentType->getType());

        $body = $message->getBody();
        $parts = array_values($body->getParts());

        $this->assertCount(2, $parts);
        $this->assertEquals('text/plain', $parts[0]->getType());
        $this->assertEquals('multipart/related', $parts[1]->getType());

        if (TestConfigProvider::SEND_EMAILS) {
            $this->sendMessage($message, __METHOD__);
        }
    }

    /**
     * @throws ContainerExceptionInterface
     * @throws NotFoundExceptionInterface
     */
    public function testCanSendAlternativeMessagesWithHtmlWithInlineImagesAndAttachments()
    {
        $flowers = __DIR__ . '/data/flowers.jpg';
        $obama = __DIR__ . '/data/obama.jpg';

        $html = <<< EOH
<table>
    <tbody>
        <tr>
            <td>
                <img src="$obama" width="200" alt=""><br>
                Neka slika
            </td>
            
            <td style="width:100%;">
                Prvi red
                <img src="$flowers" width="300" style="text-align:right;" alt="">
            </td>
        </tr>
        
        <tr>            
            <td>
                Drugi red
                <img src="$flowers" width="300" style="text-align:right;" alt="">
            </td>
            
            <td>
                <img src="$obama" width="200" alt=""><br>
                Neka slika
            </td>
        </tr>
    </tbody>
</table>
EOH;

        $message = new Message();
        $message->setBody('običan tekst');
        $message->setBody($html);

        $spreadsheet = new File(fopen(__DIR__ . '/data/spreadsheet.xlsx', 'r'), 'spreadsheet.xlsx');
        $message->addAttachment($spreadsheet);

        $headers = $message->getHeaders();

        /** @var ContentType $contentType */
        $contentType = $headers->get('Content-Type');
        $this->assertEquals('multipart/mixed', $contentType->getType());

        $body = $message->getBody();
        $parts = array_values($body->getParts());

        $this->assertCount(2, $parts);
        $this->assertEquals('multipart/alternative', $parts[0]->getType());
        $this->assertEquals($spreadsheet->getMimeType(), $parts[1]->getType());

        if (TestConfigProvider::SEND_EMAILS) {
            $this->sendMessage($message, __METHOD__);
        }
    }
}
